@@ -10,8 +10,8 @@
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
@@ -31,12 +31,8 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-systemd.h>
 
-#ifdef HAVE_SYSTEMD
-#ifdef ENABLE_SYSTEMD_JOURNAL
 #include <systemd/sd-journal.h>
-#endif
 #include <systemd/sd-daemon.h>
-#endif
 
 #include "gsm-autostart-app.h"
 #include "gsm-util.h"
@@ -961,7 +957,6 @@ app_launched (GAppLaunchContext *ctx,
                                    NULL, NULL, NULL);
 }
 
-#ifdef ENABLE_SYSTEMD_JOURNAL
 static void
 on_child_setup (GsmAutostartApp *app)
 {
@@ -988,7 +983,6 @@ on_child_setup (GsmAutostartApp *app)
                 close (standard_error);
         }
 }
-#endif
 
 static gboolean
 autostart_app_start_spawn (GsmAutostartApp *app,
@@ -1039,12 +1033,10 @@ autostart_app_start_spawn (GsmAutostartApp *app,
                 g_app_launch_context_setenv (ctx, "DESKTOP_AUTOSTART_ID", startup_id);
         }
 
-#ifdef ENABLE_SYSTEMD_JOURNAL
         if (sd_booted () > 0) {
                 child_setup_func = (GSpawnChildSetupFunc) on_child_setup;
                 child_setup_data = app;
         }
-#endif
 
         handler = g_signal_connect (ctx, "launched", G_CALLBACK (app_launched), app);
         success = g_desktop_app_info_launch_uris_as_manager (priv->app_info,
