@@ -9,8 +9,8 @@
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
@@ -239,7 +239,7 @@ screensaver_get_active_cb (GDBusProxy  *screensaver_proxy,
         gboolean is_active;
 
         data = g_dbus_proxy_call_finish (screensaver_proxy, res, &error);
-        if (data) {
+        if (!data) {
                 if (error) {
                         g_warning ("Could not retrieve current screensaver active state: %s",
                                    error->message);
