@@ -9,8 +9,8 @@
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
@@ -61,22 +61,30 @@ save_one_client (char            *id,
 {
         GsmClient  *client;
         GKeyFile   *keyfile;
-        GsmApp     *app;
+        GsmApp     *app = NULL;
         const char *app_id;
         char       *path = NULL;
         char       *filename = NULL;
         char       *contents = NULL;
         gsize       length = 0;
         char       *discard_exec;
-        GError     *local_error;
+        g_autoptr(GError) local_error = NULL;
 
         client = GSM_CLIENT (object);
 
-        local_error = NULL;
+        app_id = gsm_client_peek_app_id (client);
+        if (!IS_STRING_EMPTY (app_id)) {
+                if (g_str_has_suffix (app_id, ".desktop"))
+                        filename = g_strdup (app_id);
+                else
+                        filename = g_strdup_printf ("%s.desktop", app_id);
 
-        app = (GsmApp *)gsm_store_find (data->app_store,
-                                        (GsmStoreFunc)_app_has_app_id,
-                                        (char *)app_id);
+                path = g_build_filename (data->dir, filename, NULL);
+
+                app = (GsmApp *)gsm_store_find (data->app_store,
+                                                (GsmStoreFunc)_app_has_app_id,
+                                                (char *)app_id);
+        }
         keyfile = gsm_client_save (client, app, &local_error);
 
         if (keyfile == NULL || local_error) {
@@ -89,16 +97,6 @@ save_one_client (char            *id,
                 goto out;
         }
 
-        app_id = gsm_client_peek_app_id (client);
-        if (!IS_STRING_EMPTY (app_id)) {
-                if (g_str_has_suffix (app_id, ".desktop"))
-                        filename = g_strdup (app_id);
-                else
-                        filename = g_strdup_printf ("%s.desktop", app_id);
-
-                path = g_build_filename (data->dir, filename, NULL);
-        }
-
         if (!path || g_file_test (path, G_FILE_TEST_EXISTS)) {
                 if (filename)
                         g_free (filename);
@@ -141,8 +139,7 @@ out:
 
         /* in case of any error, stop saving session */
         if (local_error) {
-                g_propagate_error (data->error, local_error);
-                g_error_free (local_error);
+                g_propagate_error (data->error, g_steal_pointer (&local_error));
 
                 return TRUE;
         }
@@ -177,6 +174,8 @@ gsm_session_save (GsmStore  *client_store,
         data.dir = save_dir;
         data.discard_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                    g_free, NULL);
+        data.app_store = app_store;
+
         /* remove old saved session */
         gsm_session_clear_saved_session (save_dir, data.discard_hash);
         data.error = error;
