@@ -10,8 +10,8 @@
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
@@ -43,14 +43,11 @@
 #include "gsm-system.h"
 #include "gsm-fail-whale.h"
 
-#ifdef ENABLE_SYSTEMD_JOURNAL
 #include <systemd/sd-journal.h>
-#endif
 
 #define GSM_DBUS_NAME "org.gnome.SessionManager"
 
 static gboolean systemd_service = FALSE;
-static gboolean use_systemd = USE_SYSTEMD_SESSION;
 static gboolean failsafe = FALSE;
 static gboolean show_version = FALSE;
 static gboolean debug = FALSE;
@@ -283,12 +280,13 @@ initialize_gio (void)
         }
 }
 
-#ifdef ENABLE_SYSTEMD_SESSION
 static gboolean
 leader_term_or_int_signal_cb (gpointer data)
 {
         gint fifo_fd = GPOINTER_TO_INT (data);
 
+        g_debug ("Session termination requested");
+
         /* Start a shutdown explicitly. */
         gsm_util_start_systemd_unit ("gnome-session-shutdown.target", "replace-irreversibly", NULL);
 
@@ -309,6 +307,105 @@ leader_term_or_int_signal_cb (gpointer data)
         return G_SOURCE_REMOVE;
 }
 
+static void
+graphical_session_pre_state_changed_cb (GDBusProxy *proxy,
+                                        GVariant   *changed_properties)
+{
+        const char *state;
+        g_autoptr (GVariant) value = NULL;
+
+        value = g_variant_lookup_value (changed_properties, "ActiveState", NULL);
+
+        if (value == NULL)
+                return;
+
+        g_variant_get (value, "&s", &state);
+        if (g_strcmp0 (state, "inactive") == 0) {
+                g_debug ("Session services now inactive, quitting");
+                gsm_quit ();
+                return;
+        }
+}
+
+static gboolean
+monitor_hangup_cb (int          fd,
+                   GIOCondition condition,
+                   gpointer     user_data)
+{
+        g_autoptr (GDBusConnection) connection = NULL;
+        g_autoptr (GVariant) unit = NULL;
+        g_autoptr (GVariant) value = NULL;
+        g_autoptr (GError) error = NULL;
+        GDBusProxy *proxy = NULL;
+        const char *unit_path = NULL;
+
+        g_debug ("Services have begun stopping, waiting for them to finish stopping");
+
+        connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+
+        if (!connection) {
+                g_debug ("Could not get bus connection: %s", error->message);
+                gsm_quit ();
+                return G_SOURCE_REMOVE;
+        }
+
+        unit = g_dbus_connection_call_sync (connection,
+                                            "org.freedesktop.systemd1",
+                                            "/org/freedesktop/systemd1",
+                                            "org.freedesktop.systemd1.Manager",
+                                            "GetUnit",
+                                            g_variant_new ("(s)", "graphical-session-pre.target"),
+                                            G_VARIANT_TYPE ("(o)"),
+                                            G_DBUS_CALL_FLAGS_NONE,
+                                            -1,
+                                            NULL,
+                                            &error);
+        if (!unit) {
+                g_debug ("Could not get unit for graphical-session-pre.target: %s", error->message);
+                gsm_quit ();
+                return G_SOURCE_REMOVE;
+        }
+
+        g_variant_get (unit, "(&o)", &unit_path);
+
+        proxy = g_dbus_proxy_new_sync (connection,
+                                       G_DBUS_PROXY_FLAGS_NONE,
+                                       NULL,
+                                       "org.freedesktop.systemd1",
+                                       unit_path,
+                                       "org.freedesktop.systemd1.Unit",
+                                       NULL,
+                                       &error);
+        if (!proxy) {
+                g_debug ("Could not get proxy for graphical-session-pre.target unit: %s", error->message);
+                gsm_quit ();
+                return G_SOURCE_REMOVE;
+        }
+
+        value = g_dbus_proxy_get_cached_property (proxy, "ActiveState");
+
+        if (value) {
+                const char *state;
+
+                g_variant_get (value, "&s", &state);
+
+                if (g_strcmp0 (state, "inactive") == 0) {
+                        g_debug ("State of graphical-session-pre.target unit already inactive quitting");
+                        gsm_quit ();
+                        return G_SOURCE_REMOVE;
+                }
+                g_debug ("State of graphical-session-pre.target unit is '%s', waiting for it to go inactive", state);
+        } else {
+                g_debug ("State of graphical-session-pre.target unit is unknown, waiting for it to go inactive");
+        }
+        g_signal_connect (proxy,
+                          "g-properties-changed",
+                          G_CALLBACK (graphical_session_pre_state_changed_cb),
+                          NULL);
+
+        return G_SOURCE_REMOVE;
+}
+
 /**
  * systemd_leader_run:
  *
@@ -374,12 +471,13 @@ systemd_leader_run(void)
                         close (fifo_fd);
                         fifo_fd = -1;
                 } else {
-                        g_unix_fd_add (fifo_fd, G_IO_HUP, (GUnixFDSourceFunc) gsm_quit, NULL);
+                        g_unix_fd_add (fifo_fd, G_IO_HUP, (GUnixFDSourceFunc) monitor_hangup_cb, NULL);
                 }
         } else {
                 g_warning ("Unable to watch systemd session: Opening FIFO failed with %m");
         }
 
+        g_unix_signal_add (SIGHUP, leader_term_or_int_signal_cb, GINT_TO_POINTER (fifo_fd));
         g_unix_signal_add (SIGTERM, leader_term_or_int_signal_cb, GINT_TO_POINTER (fifo_fd));
         g_unix_signal_add (SIGINT, leader_term_or_int_signal_cb, GINT_TO_POINTER (fifo_fd));
 
@@ -387,7 +485,6 @@ systemd_leader_run(void)
         gsm_main ();
         exit(0);
 }
-#endif /* ENABLE_SYSTEMD_SESSION */
 
 int
 main (int argc, char **argv)
@@ -402,11 +499,7 @@ main (int argc, char **argv)
         guint             name_owner_id;
         GOptionContext   *options;
         static GOptionEntry entries[] = {
-#ifdef ENABLE_SYSTEMD_SESSION
                 { "systemd-service", 0, 0, G_OPTION_ARG_NONE, &systemd_service, N_("Running as systemd service"), NULL },
-                { "systemd", 0, 0, G_OPTION_ARG_NONE, &use_systemd, N_("Use systemd session management"), NULL },
-#endif
-                { "builtin", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &use_systemd, N_("Use builtin session management (rather than the systemd based one)"), NULL },
                 { "autostart", 'a', 0, G_OPTION_ARG_STRING_ARRAY, &override_autostart_dirs, N_("Override standard autostart directories"), N_("AUTOSTART_DIR") },
                 { "session", 0, 0, G_OPTION_ARG_STRING, &opt_session_name, N_("Session to use"), N_("SESSION_NAME") },
                 { "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
@@ -460,7 +553,6 @@ main (int argc, char **argv)
          * journald picks ups the nicer "gnome-session" as the program
          * name instead of whatever shell script GDM happened to use.
          */
-#ifdef ENABLE_SYSTEMD_JOURNAL
         if (!debug) {
                 int journalfd;
 
@@ -470,7 +562,6 @@ main (int argc, char **argv)
                         dup2(journalfd, 2);
                 }
         }
-#endif
 
         gdm_log_init ();
         gdm_log_set_debug (debug);
@@ -545,15 +636,12 @@ main (int argc, char **argv)
 
         session_name = opt_session_name;
 
-#ifdef HAVE_SYSTEMD
         gsm_util_export_user_environment (&error);
         if (error && !g_getenv ("RUNNING_UNDER_GDM"))
                 g_warning ("Failed to upload environment to systemd: %s", error->message);
         g_clear_error (&error);
-#endif
 
-#ifdef ENABLE_SYSTEMD_SESSION
-        if (use_systemd && !systemd_service) {
+        if (!systemd_service) {
                 const gchar *session_type = NULL;
                 g_autofree gchar *gnome_session_target = NULL;
 
@@ -600,7 +688,6 @@ main (int argc, char **argv)
                         exit (1);
                 }
         }
-#endif /* ENABLE_SYSTEMD_SESSION */
 
         {
                 gchar *ibus_path;
@@ -637,9 +724,7 @@ main (int argc, char **argv)
 
         gsm_main ();
 
-#ifdef HAVE_SYSTEMD
         gsm_util_export_user_environment (NULL);
-#endif
 
         g_clear_object (&manager);
         g_free (gl_renderer);
